@@ -34,6 +34,7 @@ from google.cloud.exceptions import NotFound
 from google.cloud.storage._helpers import _get_storage_host
 from google.cloud.storage._helpers import _DEFAULT_STORAGE_HOST
 from google.cloud.storage._helpers import _bucket_bound_hostname_url
+from google.cloud.storage._helpers import _add_etag_match_headers
 from google.cloud.storage._http import Connection
 from google.cloud.storage._signing import (
     get_expiration_seconds_v4,
@@ -967,6 +968,8 @@ class Client(ClientWithProject):
         start=None,
         end=None,
         raw_download=False,
+        if_etag_match=None,
+        if_etag_not_match=None,
         if_generation_match=None,
         if_generation_not_match=None,
         if_metageneration_match=None,
@@ -996,16 +999,22 @@ class Client(ClientWithProject):
             raw_download (bool):
                 (Optional) If true, download the object without any expansion.
 
-            if_generation_match: long
+            if_etag_match (Union[str, Set[str]]):
+                (Optional) See :ref:`using-if-etag-match`
+
+            if_etag_not_match (Union[str, Set[str]]):
+                (Optional) See :ref:`using-if-etag-not-match`
+
+            if_generation_match (long):
                 (Optional) See :ref:`using-if-generation-match`
 
-            if_generation_not_match: long
+            if_generation_not_match (long):
                 (Optional) See :ref:`using-if-generation-not-match`
 
-            if_metageneration_match: long
+            if_metageneration_match (long):
                 (Optional) See :ref:`using-if-metageneration-match`
 
-            if_metageneration_not_match: long
+            if_metageneration_not_match (long):
                 (Optional) See :ref:`using-if-metageneration-not-match`
 
             timeout ([Union[float, Tuple[float, float]]]):
@@ -1091,6 +1100,9 @@ class Client(ClientWithProject):
         )
         headers = _get_encryption_headers(blob_or_uri._encryption_key)
         headers["accept-encoding"] = "gzip"
+        _add_etag_match_headers(
+            headers, if_etag_match=if_etag_match, if_etag_not_match=if_etag_not_match,
+        )
 
         transport = self._http
         try:
